extern crate alloc;
use alloc::vec::*;
use spin::Mutex;

pub static LOGGER: Mutex<Logger> = Mutex::new(Logger::new());

pub struct EnumParseError {}

pub struct Logger {
    pub level: LogLevel,
    pub subscriber: Vec<alloc::boxed::Box<dyn LogSubscriber + Send + Sync>>,
}

pub trait LogSubscriber {
    fn write(&self, msg: &str);
}

impl Logger {
    pub const fn new() -> Self {
        Logger {
            level: LogLevel::WARNING,
            subscriber: Vec::new(),
        }
    }
    #[allow(unused_variables)]
    pub fn log(&self, level: LogLevel, msg: &str) {}
}

#[derive(PartialEq, Eq, PartialOrd, Ord)]
pub enum LogLevel {
    DISABLED,
    CRITICAL,
    ERROR,
    WARNING,
    INFO,
    TRACE,
}

impl TryFrom<u8> for LogLevel {
    type Error = EnumParseError;
    fn try_from(from: u8) -> Result<Self, Self::Error> {
        match from {
            0 => Ok(Self::DISABLED),
            1 => Ok(Self::CRITICAL),
            2 => Ok(Self::ERROR),
            3 => Ok(Self::WARNING),
            4 => Ok(Self::INFO),
            5 => Ok(Self::TRACE),
            _ => Err(EnumParseError {}),
        }
    }
}

impl TryFrom<&str> for LogLevel {
    type Error = EnumParseError;
    fn try_from(from: &str) -> Result<Self, Self::Error> {
        match from.to_ascii_lowercase().as_ref() {
            "disabled" => Ok(Self::DISABLED),
            "critical" => Ok(Self::CRITICAL),
            "error" => Ok(Self::ERROR),
            "warning" => Ok(Self::WARNING),
            "info" => Ok(Self::INFO),
            "trace" => Ok(Self::TRACE),
            _ => Err(EnumParseError {}),
        }
    }
}
